/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.serversmodel;

import java.io.File;
import java.util.ArrayList;
import java.util.Observable;
import java.util.Observer;

import org.gvsig.publish.IPublishPersistence;
import org.gvsig.publish.PublishLogger;
import org.gvsig.publish.PublishRegister;
import org.gvsig.publish.infoproject.IDataSourceInfo;
import org.gvsig.publish.infoproject.ILayerInfo;
import org.gvsig.publish.infoproject.IProjectInfo;
import org.gvsig.publish.infoproject.datasources.IFileInfo;

import com.iver.utiles.XMLEntity;
/**
 * Represents a publication, that is, a project information that can be accessible through a server  
 * 
 * @author jvhigon
 *
 */
public class Publication extends Observable implements IPublishPersistence, Observer {
	//Access methods
	public final int LOCAL_ACCESS=0;
	public final int REMOTE_MOUNT_POINT_ACCESS=1;

	/*
	 * Associations
	 */
	private Server server = null;
	private IProjectInfo iproject = null;
	/*
	 * Properties
	 */	
	private String title = null;
	private int accessFileMethod = LOCAL_ACCESS;
	private String localMountPoint;
	private String remoteMountPoint;

	/**
	 * Sets the name of the publication
	 * @param name
	 */
	public void setTitle(String title){
		this.title = title;
		setChanged();
		notifyObservers();
	}
	/**
	 * Gets the publication's name
	 * @param name
	 * @return
	 */
	public String getTitle(){
		return title;
	}
	/**
	 * Sets the server of the publication
	 * TODO: Is not clear. Should I use the method addObserver here? 
	 * @param s Server 
	 */
	public void setServer(Server s) {
		//set this like a observer the first time
		if (server == null){
			s.addObserver(this);
		}			
		//set associations
		s.setPublication(this);
		//init variables
		this.server = s;
		//this.server.setProjectInfo(getProjectInfo());

		//notify publication observers
		setChanged();
		notifyObservers();	
	}
	/**
	 * Gets the server of the publication
	 * @return
	 */
	public Server getServer(){
		return server;
	}
	/**
	 * @return name of the publication
	 */
	public String toString(){
		return getTitle();
	}

	/**
	 * Sets the information about the project to publish.
	 * If a remote resource hasn't layerinfo it will be 
	 * removed from the publication
	 * 
	 * @param projectInfo
	 * @return an array of strings with the name of the layers 
	 * that have been renamed or deleted
	 */
	public ArrayList setProjectInfo(IProjectInfo projectInfo) {
		iproject = projectInfo;
		ArrayList aux = new ArrayList();
		Server s = getServer();
		if (s !=null){
			for (int i = 0; i < s.getServicesCount(); i++){
				Service ss = s.getService(i);
				for (int j = 0; j < ss.getRemoteResourcesCount(); j++ ){
					RemoteResource rr = ss.getRemoteResource(j);
					ILayerInfo ilayerinfo=null;				
					ilayerinfo = projectInfo.findLayer(rr.getId());				
					if (ilayerinfo == null){						
						aux.add(rr.getId());
						rr.remove();
						PublishLogger.getLog().error("Cannot find the layer with name " + rr.getId());
					}
				}
			}
		}
		return aux;
	}
	/**
	 * 
	 * @return information about the project resources to publish
	 */
	public IProjectInfo getProjectInfo() {
		return iproject;
	}
	/*
	 * (non-Javadoc)
	 * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
	 */
	public void update(Observable o, Object arg) {
		if (o instanceof Server){
			setServer((Server)o);
		}else{
			PublishLogger.getLog().error("ERROR Publication: The observable object must be a server");
		}

	}

	/*
	 * (non-Javadoc)
	 * @see com.iver.utiles.IPersistance#getXMLEntity()
	 */
	public XMLEntity getXMLEntity() {
		XMLEntity xml=new XMLEntity();
		//put version and name		
		xml.setName(getClassName());
		xml.putProperty("version", getVersion());
		//put properties
		xml.putProperty("title", getTitle());
		xml.putProperty("accessfilemethod", getAccessFileMethod());
		xml.putProperty("localmountpoint", getLocalMountPoint());
		xml.putProperty("remotemountpoint", getRemoteMountPoint());

		if (getServer() != null){
			xml.addChild(getServer().getXMLEntity());
		}
		return xml;
	}
	/*
	 * (non-Javadoc)
	 * @see com.iver.utiles.IPersistance#setXMLEntity(com.iver.utiles.XMLEntity)
	 */
	public void setXMLEntity(XMLEntity xml) {
		//check version
		int version = xml.getIntProperty("version");		
		if (version != getVersion()){
			PublishLogger.getLog().error("ERROR: " + getVersion() + ": the version doesn't match!");
			return;
		}
		//set properties		
		setAccessFileMethod(xml.getIntProperty("accessfilemethod"));
		setLocalMountPoint(xml.getStringProperty("localmountpoint"));
		setRemoteMountPoint(xml.getStringProperty("remotemountpoint"));
		//creates only the first server if exists
		if (xml.getChildrenCount() > 0){
			XMLEntity xmlChild=xml.getChild(0);		
			String server_type = xmlChild.getStringProperty("registertag");
			Server s = PublishRegister.register().getServer(server_type);		
			if (s==null){			
				PublishLogger.getLog().error("ERROR " + getVersion()+ ": I can't create the server from persistence");
			}else{
				//s.setProjectInfo(iproject);			
				s.setXMLEntity(xmlChild);
				setServer(s);
			}	
		}

	}
	/*
	 * (non-Javadoc)
	 * @see com.iver.utiles.IPersistance#getClassName()
	 */
	public String getClassName() {		
		return "Publication_v1";
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.IPublishPersistence#getVersion()
	 */
	public int getVersion() {
		return 1;
	}
	/**
	 * Gets the method in which the publication has been set. 
	 * It can be Publication.LOCAL_ACCESS or Publication.REMOTE_MOUNT_POINT_ACCESS <p>
	 * LOCAL_ACCESS: the files to publish are located in the local filesystem. <br>
	 * REMOTE_MOUNT_POINT_ACCESS: the files are located in remote filesystem but are accessible through a remote mount point  
	 * 
	 * @return the accessFileMethod
	 */
	public int getAccessFileMethod() {
		return accessFileMethod;
	}
	/**
	 * Sets the method in which the files are been accessed. It can be Publication.LOCAL_ACCESS or Publication.REMOTE_MOUNT_POINT_ACCESS.
	 * LOCAL_ACCESS: the files to publish are located in the local filesystem. <br>
	 * REMOTE_MOUNT_POINT_ACCESS: the files are located in remote filesystem but are accessible through a remote mount point   
	 * @param accessFileMethod the accessFileMethod to set
	 */
	public void setAccessFileMethod(int accessFileMethod) {		
		this.accessFileMethod = accessFileMethod;
//		if (accessFileMethod == LOCAL_ACCESS){
//		setLocalMountPoint(null);
//		setRemoteMountPoint(null);
//		}
		setChanged();
	}

	/**
	 * If the publication uses a REMOTE_MOUNT_POINT_ACCESS, this method change the path to a file in order
	 * to be accessible by a server. 
	 * 
	 * @return 
	 */
	public String getServerSideFilePath(IDataSourceInfo ds){
		String aux = ((IFileInfo)ds).getAbsolutePath();			
		return getServerSideFilePath(aux);		
	}
	/**
	 * If the publication uses a REMOTE_MOUNT_POINT_ACCESS, this method change the path to a file in order
	 *
	 * @param path changed
	 * @return
	 */
	public String getServerSideFilePath(String path){		
		if (getAccessFileMethod() == REMOTE_MOUNT_POINT_ACCESS){	
			if(remoteMountPoint.charAt(remoteMountPoint.length()-1)!= '/'){
				remoteMountPoint = remoteMountPoint + "/";
			}
			path = path.replace(localMountPoint, remoteMountPoint);
			path = path.replace(File.separator, "/");
		}	
		return  path;		
	}

	/**
	 * @param localMountPoint the localMountPoint to set
	 */
	public void setLocalMountPoint(String localMountPoint) {				
		this.localMountPoint = localMountPoint;
		setChanged();	
	}
	/**
	 * @param remoteMountPoint the remoteMountPoint to set
	 */
	public void setRemoteMountPoint(String remoteMountPoint) {				
		this.remoteMountPoint = remoteMountPoint;
		setChanged();
	}
	/**
	 * @return the localMountPoint
	 */
	public String getLocalMountPoint() {
		return localMountPoint;
	}
	/**
	 * @return the remoteMountPoint
	 */
	public String getRemoteMountPoint() {
		return remoteMountPoint;
	}
}
