/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.serversmodel;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Set;

import org.gvsig.publish.IPublishPersistence;
import org.gvsig.publish.PublishLogger;
import org.gvsig.publish.PublishRegister;
import org.gvsig.publish.exceptions.PublishException;
import org.gvsig.publish.infoproject.ILayerInfo;

import com.iver.utiles.XMLEntity;

/**
 * This class represents a remote resource
 * 
 * @author Jos Vicente Hign (josevicente.higon@iver.es)
 *
 */
public abstract class RemoteResource implements IPublishPersistence {
	//associations
	private LinkedHashMap childs = null;
	protected ILayerInfo layerInfo = null;
	private Service service=null;
	//attributes
	private String id = null;

	/**
	 * Constructor
	 * 
	 * Only creates a empty hashMap for its childs 
	 *  
	 */
	public RemoteResource(Service service){
		childs = new LinkedHashMap();
		this.service = service;
	}
	/**
	 * 
	 * @return unique identifier which identifies the remote resource, It's the layerinfo name 
	 * 
	 */
	public String getId(){
		if (id == null){			
				this.id = getLayerInfo().getName();			
		}
		return id;
	}
	private void setId(String id){
		this.id = id;
	}
	/**
	 * 
	 * @return string to register 
	 */
	public abstract String getRegisterTag();

	/**
	 * 
	 * @return the service which serves this remote resource
	 */
	public Service getService(){
		return this.service;
	}
	/**
	 * Preconditions: exists information about the layer. The availability of the information is checked when a 
	 * project information is set to the publication 
	 * 
	 * @return the layer which represents this remote resource. I can be null if the remote resource has not a layerinfo associated.
	 *   
	 */
	public ILayerInfo getLayerInfo() {
		if (layerInfo == null){
			layerInfo = getService().getServer().getPublication().getProjectInfo().findLayer(getId());			
		}
		return this.layerInfo;
	}
	/**
	 * 
	 * @return number of remote resources childs
	 */
	public int getChildsCount(){
		return childs.size();
	}

	/**
	 * 
	 * @param position
	 * @return the remote resource added in the position specified by the param
	 */
	public RemoteResource getChild(int position){
		Collection collection = childs.values();
		Object[] objects = collection.toArray();
		return (RemoteResource) objects[position];		
	}
	/**
	 * Add a remoteresource child
	 * @param r
	 */
	public void addChild(RemoteResource r){
		childs.put(r.getId(), r);
	}
	/**
	 * Removes itself from its service
	 */
	public void remove(){

		getService().removeRemoteResource(this);
	}
	/**
	 * Call the method publish of each child
	 * @throws PublishException
	 * @throws LayerInfoNotAvailableException 
	 */
	public void publish() throws PublishException{
		//Call childs publish method 
		Collection allRemoteResources = childs.values();
		Iterator i = allRemoteResources.iterator();
		while(i.hasNext()){
			RemoteResource r = (RemoteResource) i.next();
			r.publish();
		}		
	}

	/**
	 * Initializes the remote resource with a layer info.
	 * Creates a new remote resource child for each layerinfo child
	 * 
	 * @param layerInfo
	 */

	public void setLayerInfo(ILayerInfo ilayer) {
		//TODO: clear all its childs?
		//childs.clear();
		//assign the reference to infolayer
		layerInfo = ilayer;
		//creates its childs
		ILayerInfo[] layerInfos = layerInfo.getChilds();
		if (layerInfos != null){
			for (int i = 0; i < layerInfos.length; i++){			
				RemoteResource child = null;
				Set set = PublishRegister.register().getRemoteResourcesNames(getService().getServer().getRegisterTag(),getService().getRegisterTag());
				//At the moment only one remote resource by service
				String remoteResourceName = (String)set.iterator().next();
				child = PublishRegister.register().getRemoteResource(getService(),remoteResourceName);				
				//TODO: check problems in initialize
				child.setLayerInfo(layerInfos[i]);
				childs.put(layerInfos[i].getName(), child);
			}
		} 
	}

	/**
	 * Puts all the properties into a XML in order to persist it
	 * 
	 * @return
	 */
	public XMLEntity getXMLEntity() {
		XMLEntity xml=new XMLEntity();
		//put the version?? NO because is abstract
		//put properties
		xml.putProperty("registertag", getRegisterTag());
		xml.putProperty("id", getId());
		//put associations 		
		XMLEntity xmlRemotes = new XMLEntity();
		xmlRemotes.setName("children");
		for (int i = 0; i< getChildsCount(); i++){
			RemoteResource child = getChild(i);
			xmlRemotes.addChild(child.getXMLEntity());
		}
		xml.addChild(xmlRemotes);
		return xml;
	}
	/**
	 * Initializes the remote resource from the persistence
	 * @param xml
	 */
	public void setXMLEntity(XMLEntity xml) {					
		setId(xml.getStringProperty("id"));
		XMLEntity children = xml.firstChild("name", "children");
		for (int i=0; i<children.getChildrenCount(); i++){
			XMLEntity child = children.getChild(i);
			RemoteResource r = PublishRegister.register().getRemoteResource(getService(), child.getStringProperty("registertag"));
			if (r == null){
				PublishLogger.getLog().error("ERROR RemoteResource: I can't create a child from persistence");
			}
			r.setXMLEntity(child);
			addChild(r);
		}

	}
	/**
	 * Removes the child if exists 
	 * 
	 * @param r remote resource to remove
	 */
	public void removeChild(RemoteResource r) {

		if (childs.containsKey(r.getId())){
			childs.remove(r.getId());
			if (childs.isEmpty()){
				remove();
			}
		}else{
			for (int i = 0; i < getChildsCount(); i++){
				getChild(i).removeChild(r);
			}
		}

	}

}
