/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/**
 * @author Jos Vicente Hign (josevicente.higon@iver.es)
 */
package org.gvsig.publish.serversmodel;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Observable;
import java.util.Set;

import org.gvsig.publish.IPublishPersistence;
import org.gvsig.publish.PublishLogger;
import org.gvsig.publish.PublishRegister;
import org.gvsig.publish.exceptions.InvalidRemoteResourceException;
import org.gvsig.publish.exceptions.PublishException;
import org.gvsig.publish.infoproject.ILayerInfo;
import org.gvsig.publish.infoproject.IViewInfo;

import com.iver.utiles.XMLEntity;

/**
 * Represents a server which implements services  
 * 
 * @author jvhigon
 *
 */
public abstract class Server extends Observable implements IPublishPersistence {
	 
	//Association to the publication
	private Publication publication = null;
	//Attributes
	private URL serverURL = null;
	private LinkedHashMap services = null; 

	/**
	 * 
	 * @return unique server identifier.
	 */
	public abstract String getId();
	/**
	 * 
	 * @return string to register 
	 */
	public abstract String getRegisterTag();

	/**
	 * Constructor
	 * 
	 * Creates all the services of the server
	 * 
	 */
	public Server(){
		//creates a hashmap with its services 
		services = new LinkedHashMap();		
	}
	
	/**
	 * Adds information about a view in all services
	 * @param view
	 * @throws InvalidRemoteResourceException 
	 */
	public void addInfo(IViewInfo view) throws InvalidRemoteResourceException{
		for(int i = 0; i< getServicesCount();i++){
			getService(i).addInfo(view);
		}
		//notify observers (publication)
		setChanged();
		notifyObservers();
	}
	/**
	 * Adds information about a layer in all services
	 * @param layer
	 * @throws InvalidRemoteResourceException 
	 */
	public void addInfo(ILayerInfo layer) throws InvalidRemoteResourceException{
		for(int i = 0; i< getServicesCount();i++){
			getService(i).addInfo(layer);
		}
		//notify observers (publication)
		setChanged();
		notifyObservers();

	}

	/**
	 * Gets a service identified by a name 
	 * @param serviceName
	 * @return
	 */
	public Service getService(String serviceName){
		Service s = null;
		s = (Service) services.get(serviceName);
		return s;
	}
	/**
	 * Useful for generate a tree model
	 * @return
	 */
	public Service getService(int position){
		Collection collection = services.values();
		Object[] objects = collection.toArray();
		if (objects.length == 0 ){
			return null;
		}else{
			return (Service) objects[position];
		}
	}
	/**
	 * 
	 * @return number of services in the server
	 * TODO: Why not a services.size()???
	 */
	public int getServicesCount(){
		Set set = services.entrySet();
		Object[] objects = set.toArray();
		return objects.length;
	}
	/**
	 * Adds a service into the server
	 * @param serviceName
	 * @param service
	 */
	public void addService(Service service){
		services.put(service.getId(), service);
	}


	/**
	 * 
	 * Call the method publish of its services
	 * @throws PublishException
	 */
	public String publish() throws PublishException{
		Collection allServices = services.values();
		Iterator i = allServices.iterator();
		while(i.hasNext()){
			Service s = (Service) i.next();
			s.publish();
		}		
		return "";
	}
	/**
	 * Each server must verify if its remote resources are compatible
	 */
	public abstract void postInitialize();

	/**
	 * 
	 * @return server's url
	 */
	public URL getServerURL() {
		return serverURL;
	}
	/**
	 * Sets the server's url
	 * @param serverURL
	 */
	public void setServerURL(URL serverURL) {
		this.serverURL = serverURL;
	}
	
	/**
	 * Puts all the properties into a XML in order to persist it
	 * @return
	 */
	public XMLEntity getXMLEntity() {
		XMLEntity xml=new XMLEntity();
		//put server properties				
		xml.putProperty("registertag", getRegisterTag());
		xml.putProperty("id", getId());
		xml.putProperty("url", getServerURL().toString());
		//put associations (its services)
		XMLEntity children=new XMLEntity();
		children.setName("services");
		for (int i=0;i<getServicesCount();i++){
			children.addChild(getService(i).getXMLEntity());
		}
		xml.addChild(children);
		return xml;
	}
	/**
	 * Initializes from the persistence
	 * @param xml
	 */	
	public void setXMLEntity(XMLEntity xml) {
		//set properties		
		String urlString = xml.getStringProperty("url");
		URL url=null;
		URL aux=null;
		try {
			url = new URL(urlString);
			//auxiliary well formed url
			aux = new URL("http://");
		} catch (MalformedURLException e1) {
			PublishLogger.getLog().error("ERROR Server: Malformed url setting the server url from persistence");
			setServerURL(aux);
		}
		setServerURL(url);
		//set all its services
		XMLEntity children = xml.firstChild("name", "services");
		for (int i=0;i<children.getChildrenCount();i++){
			XMLEntity xmlChild=children.getChild(i);
			String service_type = xmlChild.getStringProperty("registertag");
			Service service=PublishRegister.register().getService(this,service_type);
			service.setXMLEntity(xmlChild);
			addService(service);
		}
	}
	/**
	 * Removes a service from the server
	 * @param service to remove
	 */
	public void removeService(Service service) {
		services.remove(service.getId());
		
	}
	/**
	 * @return the publication related to the server
	 */
	public Publication getPublication() {
		return publication;
	}
	/**
	 * @param publication, the publication related to the server
	 */
	public void setPublication(Publication publication) {
		this.publication = publication;
	} 

}
