/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.serversmodel;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Observable;
import java.util.Set;

import org.gvsig.publish.IPublishPersistence;
import org.gvsig.publish.PublishLogger;
import org.gvsig.publish.PublishRegister;
import org.gvsig.publish.exceptions.InvalidRemoteResourceException;
import org.gvsig.publish.exceptions.PublishException;
import org.gvsig.publish.infoproject.FilterProjectInfo;
import org.gvsig.publish.infoproject.ILayerInfo;
import org.gvsig.publish.infoproject.IViewInfo;

import com.iver.utiles.XMLEntity;

/**
 * A server implements severals services.
 * 
 * @author jvhigon
 *
 */
public abstract class Service extends Observable implements IPublishPersistence {

	/*
	 * HasMap with its remote resources
	 */
	private LinkedHashMap remoteResources = null;
	//Associations
	private Server server = null;

	/**
	 * Constructor
	 * Only creates a empty hashmap with its remote resources 
	 */
	public Service(Server server){
		remoteResources = new LinkedHashMap();
		this.server = server;
	}
	/**
	 * 
	 * @return string which identifies the service in the publish register
	 */
	public abstract String getRegisterTag();
	/**
	 * 
	 * @return unique identifier which identifies a service instance 
	 */
	public abstract String getId();
	/**
	 * 
	 * @return Filter to apply to project information 
	 */
	public abstract FilterProjectInfo getFilter();
	

	/**
	 * 
	 * @return the server which implements that service
	 */
	public Server getServer(){
		return server;
	}	
	/**
	 * 
	 * @throws PublishException
	 */
	public void publish() throws PublishException{
		Collection allRemoteResources = remoteResources.values();
		Iterator i = allRemoteResources.iterator();
		while(i.hasNext()){
			RemoteResource r = (RemoteResource) i.next();
			r.publish();
		}		
	}
	/**
	 * 
	 * @return number of remote resources in the service
	 */
	public int getRemoteResourcesCount(){
		return remoteResources.size();
	}
	/**
	 * 
	 * @param position
	 * @return the remote resource added in the position specified by the parameter
	 */
	public RemoteResource getRemoteResource(int position){
		Collection collection = remoteResources.values();
		Object[] objects = collection.toArray();
		return (RemoteResource) objects[position];		
	}
	/**
	 * 
	 * I need an attribute name in the remote resource
	 * @throws InvalidRemoteResourceException if the remote resource already exists (same id)
	 */
	public void addRemoteResource(RemoteResource r) throws InvalidRemoteResourceException{
		if (remoteResources.containsKey(r.getId())){
			throw new InvalidRemoteResourceException("two_remoteresources_with_same_id");
		}else{
			remoteResources.put(r.getId(), r);
		}
	}
	/**
	 * Adds information about the view. For each layer info, 
	 * creates a remote resource (the first registered)
	 * This method is only useful when one service only has one remote resource type.
	 * If you have more one kind of remote resource, you must override this method.
	 * 
	 * @param view
	 * @throws InvalidRemoteResourceException if there are duplicated remote resources (same id)
	 */
	public void addInfo(IViewInfo view) throws InvalidRemoteResourceException {
		Set set = PublishRegister.register().getRemoteResourcesNames(getServer().getRegisterTag(),getRegisterTag());
		//if no remote resources registered return
		if (set ==null){
			PublishLogger.getLog().error("ERROR " + getClassName() + ":I can't add Info");
			return;
		}
		//At the moment only one remote resource by service
		String remoteResourceName = (String)set.iterator().next();

		//get all the layers and creates a new remote resource
		ILayerInfo[] layers = view.getLayers();
		for (int i=0; i< layers.length; i++){
			RemoteResource r = PublishRegister.register().getRemoteResource(this,remoteResourceName);
			r.setLayerInfo(layers[i]);
			//add the remote resource 
			addRemoteResource(r);
		}

	}
	/**
	 * Adds a layer information . Creates a remote resource if the service is in the register.
	 * 
	 * @param layer
	 * @throws InvalidRemoteResourceException  
	 */
	public void addInfo(ILayerInfo layer) throws InvalidRemoteResourceException  {
		Set set = PublishRegister.register().getRemoteResourcesNames(getServer().getRegisterTag(),getRegisterTag());
		if (set != null){
		//At the moment only one remote resource by service
		String remoteResourceName = (String)set.iterator().next();
		RemoteResource r = PublishRegister.register().getRemoteResource(this,remoteResourceName);
		//sets the layer info
		r.setLayerInfo(layer);
		//add the remote resource
		addRemoteResource(r);
		}
	}

	/**
	 * Puts all the properties into a XML in order to persist it
	 * @return
	 */
	public XMLEntity getXMLEntity() {
		XMLEntity xml=new XMLEntity();
		//put version
		//xml.setName(getVersion());
		xml.putProperty("registertag", getRegisterTag());
		xml.putProperty("id", getId());
		//put associations
		XMLEntity children=new XMLEntity();
		children.setName("remoteresources");		
		for (int i=0;i<getRemoteResourcesCount();i++){
			children.addChild(getRemoteResource(i).getXMLEntity());
		}
		xml.addChild(children);
		return xml;
	}
	/**
	 * Initializes the remote resource from the persistence
	 * @param xml
	 */
	public void setXMLEntity(XMLEntity xml) {
		//set properties
		//set children	
		XMLEntity children = xml.firstChild("name", "remoteresources");
		for (int i=0;i<children.getChildrenCount();i++){
			XMLEntity xmlChild=children.getChild(i);
			String remote_type = xmlChild.getStringProperty("registertag");
			RemoteResource remote=PublishRegister.register().getRemoteResource(this,remote_type);
			remote.setXMLEntity(xmlChild);
			try {
				addRemoteResource(remote);
			} catch (InvalidRemoteResourceException e) {
				PublishLogger.getLog().warn("WARN: skipping remote resource with id " + remote.getId() + " because it already exists", e);
			}
		}
	}
	/**
	 * Removes a remote resource. It call the method removeChild() of all its remote resources
	 * 
	 */
	public void removeRemoteResource(RemoteResource r) {
		//If the remote resource is in the root
		if (remoteResources.containsKey(r.getId())){
			remoteResources.remove(r.getId());
		}else{
			for (int i = 0; i < getRemoteResourcesCount(); i++){
				getRemoteResource(i).removeChild(r);
			}
		}
		
		
		//notify observer
		setChanged();
		notifyObservers();
	}
	/**
	 * Removes itself from its service
	 */
	public void remove() {
		getServer().removeService(this);		
	}
}
